package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.utils.StringTabularReader;

public class DemandManagerFromFile extends AbstractDemandManager {

	private StringTabularReader historicalConsumptionReader;
	
	public DemandManagerFromFile(CompositeCountryManager compositeCountryManager, CalorieManager calorieManager) {
		super(compositeCountryManager, calorieManager);
		historicalConsumptionReader = new StringTabularReader(",", new String[]{"Year", "Country", "Item", "population", "cpc"});
		historicalConsumptionReader.read(ModelConfig.DEMAND_CONSUMPTION_FILE);
	}

	@Override
	protected Map<CommodityType, Double> getFoodDemand(SingleCountry c, int year, Map<CommodityType, Double> prices) {
		Map<CommodityType, Double> foodDemandMap = new HashMap<CommodityType, Double>();

		for (CommodityType commodity : CommodityType.getAllFoodItems()) {
			Map<String, String> queryMap = new HashMap<String, String>();
			queryMap.put("Year", Integer.toString(year));
			queryMap.put("Country", c.getCountryName());
			queryMap.put("Item", commodity.getFaoName());
			try {
				Map<String, String> row = historicalConsumptionReader.querySingle(queryMap);
				String popS = row.get("population");
				String cpcS = row.get("cpc");
				Double pop = Double.valueOf(popS);
				Double cpc = Double.valueOf(cpcS);
				double d = cpc * pop * ModelConfig.SSP_POPULATION_FACTOR;
				foodDemandMap.put(commodity, d);
			}
			catch (Exception e) {
				LogWriter.println("Problem finding food demand: " + commodity.getFaoName() + ", " + year + ", " + c.getCountryName());
			}
		}
		
		if(!ModelConfig.SHOCKS_POSSIBLE){
			return foodDemandMap;
		}
		return dietaryAdjustment(foodDemandMap, year, calorieManager.get(c));
	}	
}