package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.ModelFitType;

public class DemandManagerSSP extends AbstractSSPDemandManager {

	private DemandCurveManager demandCurveManager;

	public DemandManagerSSP(String ssp_scenario, BaseConsumpManager baseConsumpManager, CalorieManager calorieManager, CompositeCountryManager compositeCountryManager) {
		super(ssp_scenario, baseConsumpManager, calorieManager, compositeCountryManager);
		demandCurveManager = new DemandCurveManager();		
	}

	@Override
	Map<CommodityType, Double> getFoodDemandMap(SingleCountry c, int year, double gdpPc, double population, Map<CommodityType, Double> prices, double usaGdpPc) {
		Map<CommodityType, Double> aFoodDemandMap = new HashMap<CommodityType, Double>();
		SspData baseSspData = sspManager.get(ssp_scenario, ModelConfig.BASE_YEAR, c);
		double baseGdpPc = baseSspData.getGdpPc();

		for (CommodityType commodity : CommodityType.getAllFoodItems()) {
			ModelFitType fitType = commodity.isAnimalProduct() ? ModelConfig.DEMAND_ANIMAL_PROD_FIT : ModelConfig.DEMAND_NON_ANIMAL_PROD_FIT;
			DemandCurve dc = demandCurveManager.get(commodity, fitType);
			double baseExpCpc = dc.getProjectConsumptionPc(baseGdpPc);
			double newExpCpc = dc.getProjectConsumptionPc(gdpPc);
			double baseCpc = baseConsumpManager.get(c, commodity);
			
			double cpc = rebaseConsumption(baseGdpPc, baseCpc, baseExpCpc, gdpPc, newExpCpc);					
			double d = cpc * population;
			aFoodDemandMap.put(commodity, d);
		}
		
		return aFoodDemandMap;
	}	
}